//@ts-check
"use strict";

/**
 * @typedef TileSet
 * @property {number} width
 * @property {number} height
 * @property {number} tileWidth
 * @property {number} tileHeight
 * @property {number} columns
 * @property {number} rows
 * @property {{ [tileId: number]: string }} tileTypesById
 * @property {{ [tileId: number]: TileSetAnimation }} animationsById
 * @property {{ [tileId: number]: TileSetCollision }} collisionsById
 * 
 * @typedef TileSetAnimation
 * @property {TileSetAnimationFrame[]} frames
 * @property {number} totalDuration
 * 
 * @typedef TileSetAnimationFrame
 * @property {number} tileId
 * @property {number} duration
 * 
 * @typedef TileSetCollision
 * @property {number} x
 * @property {number} y
 * @property {number} width
 * @property {number} height
 * @property {boolean} ignoreBullets
 */

/** @returns TileSet */
function loadTileset() {
  const tilesetJson = getJXONTree(app.loader.resources[`inGame/${mapsFolderName}/tileset.tsx`].data).tileset;


  /** @type {{ [tileId: number]: string }} */
  const tileTypesById = {};
  /** @type {{ [tileId: number]: TileSetCollision }} */
  const collisionsById = {};
  /** @type {{ [key: number]: TileSetAnimation }} */
  const animationsById = {};

  for (const tile of tilesetJson.tile) {
    if (tile["@type"] != null) tileTypesById[tile["@id"]] = tile["@type"];

    if (tile.objectgroup != null && tile.objectgroup.object != null) {
      const obj = tile.objectgroup.object;
      if (obj["@type"] != null && obj["@type"].startsWith("Hitbox")) {
        collisionsById[tile["@id"]] = {
          x: obj["@x"], y: obj["@y"], width: obj["@width"], height: obj["@height"], ignoreBullets: obj["@type"].endsWith("ignoreBullets")
        };
      }
    }

    if (tile.animation != null) {
      const frames = [];
      let totalDuration = 0;

      for (const frame of tile.animation.frame) {
        const frameDuration = frame["@duration"] / 1000;
        frames.push({ tileId: frame["@tileid"], duration: frameDuration });
        totalDuration += frameDuration;
      }

      animationsById[tile["@id"]] = { frames, totalDuration };
    }
  }

  /** @type {TileSet} */
  const tileset = {
    width: tilesetJson.image["@width"], height: tilesetJson.image["@height"],
    tileWidth: tilesetJson["@tilewidth"], tileHeight: tilesetJson["@tileheight"],
    columns: tilesetJson["@columns"], rows: tilesetJson["@tilecount"] / tilesetJson["@columns"],
    tileTypesById, collisionsById, animationsById
  };

  return tileset;
}
